//
//  WebViewControl.m
//  WebViewControl
//
//  Created by Patrick Geiller on 18/03/08.
//  Copyright 2008 __MyCompanyName__. All rights reserved.
//

#import "WebViewControl.h"

@implementation WebViewControl

@synthesize htmlPageName;


//
//	AwakeFromNib
//		load page and do some setup
//
- (void)awakeFromNib
{
	NSString* path = [[NSBundle mainBundle] pathForResource:self.htmlPageName ofType:@"html"];
	if (!path)
	{
		NSLog(@"%s : html file not found", object_getClassName(self));
		return;
	}
	NSURL* url = [NSURL fileURLWithPath:path];
	
	// We'll be our own frame load delegate and receive didFinishLoadForFrame
	[self setFrameLoadDelegate:self];
	[[self mainFrame] loadRequest:[NSURLRequest requestWithURL:url]];
	
	// Binding storage
	// in Ruby this would be a hash[bindingName] = { object => ..., keyPath => ... }
	bindingNames = [[NSMutableArray alloc] init];
	bindingObservedObjects = [[NSMutableArray alloc] init];
	bindingObservedKeyPaths = [[NSMutableArray alloc] init];
}



////////////////////////////////////////////////////////////////
//
//	Bindings and Key Value Observing
//	http://developer.apple.com/documentation/Cocoa/Conceptual/CocoaBindings/Concepts/HowDoBindingsWork.html	
//	http://homepage.mac.com/mmalc/CocoaExamples/controllers.html (GraphicsBindings)
//
//////////////////////////////////////////////////////////////////

//
//	Bind
//		record what we're being bound to, and observe its changes
//
- (void)bind:(NSString *)binding toObject:(id)observable withKeyPath:(NSString *)keyPath options:(NSDictionary *)options
{
	// What we're bound to
	[bindingNames addObject:[[NSString alloc] initWithString:binding]];
	[bindingObservedObjects addObject:observable];
	[bindingObservedKeyPaths addObject:[[NSString alloc] initWithString:keyPath]];
	
	// Observe binding source
    [observable addObserver:self                  
				forKeyPath:keyPath
				options:0
				context:nil];
				
	// Dispatch first notification
	[self observeValueForKeyPath:keyPath ofObject:observable change:nil context:nil];				
}


//
//	Key Value Observing : any object observed will trigger a call here
//
- (void)observeValueForKeyPath:(NSString *)keyPath ofObject:(id)object change:(NSDictionary *)change context:(void *)context
{
	// Get key index
	NSUInteger i;
	int idx = -1;
	for (i=0; i<[bindingObservedKeyPaths count]; i++)
	{
		NSString* bindingKeyPath = [bindingObservedKeyPaths objectAtIndex:i];
		if ([bindingKeyPath isEqualToString:keyPath])
		{
			idx = i;
			break;
		}
	}
	
	// If we don't know that key, bail
	if (idx == -1)
	{
		NSLog(@"Unknow key %@ in %s.observeValueForKeyPath", keyPath, object_getClassName(self));
		return;
	}
	
	// Dispatch new value to Javascript
	NSString* bindingName		= [bindingNames objectAtIndex:idx];
	id observedObject			= [bindingObservedObjects objectAtIndex:idx];
	id newValue					= [observedObject valueForKeyPath:keyPath];
//	NSLog(@"New value %@ %s  %@", keyPath, object_getClassName(object), bindingName);
	[self setValue:newValue forKey:bindingName];
	[[self windowScriptObject] callWebScriptMethod:@"WebViewControl_valueChanged" withArguments:[NSArray arrayWithObjects: bindingName, nil]];
}




////////////////////////////////////////////////////////////////
//
//	WebView
//
//////////////////////////////////////////////////////////////////



//
//	Frame loaded
//		As we only host one page, we'll consider that as our onload
- (void)webView:(WebView *)sender didFinishLoadForFrame:(WebFrame *)frame
{
//	NSLog(@"LOAD");
}


//
//	Javascript is available
//		Register our custom javascript object in the hosted page
//
- (void)webView:(WebView *)webView windowScriptObjectAvailable:(WebScriptObject *)windowScriptObject
{
	// Register ourselves as window['WebViewControl']
	[windowScriptObject setValue:self forKey:@"WebViewControl"];
}



//
//	Access restriction
//		Choose what methods and properties our ObjC object will expose to Javascript.
//		
+ (BOOL)isSelectorExcludedFromWebScript:(SEL)selector {
	return NO;
}
+ (BOOL)isKeyExcludedFromWebScript:(const char *)property {
	return NO;
}


//
//	(called from Javascript) 
///	Setting a new value : 
//		new value is dispatched to binding source
//
- (void)setNewValue:(id)newValue forKey:(NSString*)key
{
	// Get key index
	NSUInteger i;
	int idx = -1;
	for (i=0; i<[bindingNames count]; i++)
	{
		NSString* bindingName = [bindingNames objectAtIndex:i];
		if ([bindingName isEqualToString:key])
		{
			idx = i;
			break;
		}
	}
	// If we don't know that key, bail
	if (idx == -1)
	{
		NSLog(@"Unknow key %@ in %s.setNewValue", key, object_getClassName(self));
		return;
	}
	

	// Retrieve observed object and keyPath thanks to key
	id observedObject			= [bindingObservedObjects objectAtIndex:idx];
	NSString* observedKeyPath	= [bindingObservedKeyPaths objectAtIndex:idx];
	// Dispatch new value to binding source
	[observedObject setValue:newValue forKeyPath:observedKeyPath];
}


//
//	Don't draw background
//
- (BOOL)drawsBackground
{
	return	NO;
}

@end
